<?php
namespace Util;

use Util\Environment;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\ORM\DateTime;

class Utilities {
    const GENERAL_ENTITY_NAMESPACE = '\Teknisa\\Model\\Geral\\Entities\\';
    const TEKNISA_ENTITY_NAMESPACE = '\Teknisa\\Model\\Teknisa\\Entities\\';
    const DATE_TIME_ZONE_SP = 'America/Sao_Paulo';

    const ACCESS_CONTROL_RULE_REPOSITORY = 'ZeedhiUser\Model\AccessControl\Entities\ZhuAccessControlRule';
    const ACCESS_CONTROL_ELEMENT_REPOSITORY = 'ZeedhiUser\Model\AccessControl\Entities\ZhuAccessControlElement';

    /**
     * @param $string
     * @return string
     */
    public function encrypt($string){
        return md5($string);
    }

    /**
     * @param $password
     * @return mixed
     * @throws \Exception
     */
    public function getDelphiPasswordEncrypted($password){
        $url = 'http://security.teknisa.com/comwebservice/service.asmx?wsdl';

        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")
        );

        $params = array(
            'rdm'    => 'Bib_html.rdmDBGeralHtm',
            'method' => 'GetCriptoPass',
            'args'   => $args,
        );

        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }

        return $encryption;
    }

    /**
     * @param \DateTime $initialDate
     * @param \DateTime $endDate
     * @return int
     */
    public static function getDateIntervalInSeconds(\DateTime $initialDate, \DateTime $endDate) {
        $interval = $initialDate->diff($endDate);

        $intervalDays     = (int)$interval->format('%R%a');
        $intervalHours    = (int)$interval->format('%R%h');
        $intervalMinutes  = (int)$interval->format('%R%i');
        $intervalSeconds  = (int)$interval->format('%R%s');

        $intervalHours    = $intervalHours   + $intervalDays  * 24;
        $intervalMinutes  = $intervalMinutes + $intervalHours * 60;
        return $intervalSeconds + $intervalMinutes * 60;
    }

    public function getCurrentDate() {
        $currentDate = new DateTime();
        $currentDate->setTimezone(new \DateTimeZone(Utilities::DATE_TIME_ZONE_SP));
        return $currentDate;
    }

    public function getRandomPassword(){
        $length = 7;
        $characters = 'abcdefghijkmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890@#$';
        $charactersLength = strlen($characters);
        $randomPassword = '';
        for($i = 1; $i <= $length; $i++){
            $rand = mt_rand(1, $charactersLength);
            $randomPassword .= $characters[$rand-1];
        }
        return $randomPassword;
    }

    public static function getValueInFilterDataSourceByName(array &$filters, $nameFilter, $remove = false) {
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if ($name == $nameFilter) {
                if ($remove) {
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }

    public function callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url){
        set_time_limit(180);
        $header = array();
        $header[] = 'OAuth-Token: ' . $token;
        $header[] = 'OAuth-SecretId: ' . $privateKey;
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($data));

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url . $requestType);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        $result = curl_exec($ch);
        $result = json_decode($result, true);
        $error = curl_error($ch);
        curl_close($ch);

        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else {
            return isset($result['dataset'][$dataSetResultPos]) ? $result['dataset'][$dataSetResultPos] : null;
        }
    }

    public static function getArrayFromAssociativeArray( array $array = array(), $key = 'ID')
    {
        $result = array();
        foreach ($array as $row) {
            $result[] = $row[$key] ;
        }
        return $result;
    }

    public function zeedhiEncrypt($text, $salt = '1460_loginManagement')
    {
        return trim(
            base64_encode(
                mcrypt_encrypt(
                    MCRYPT_RIJNDAEL_256,
                    str_pad($salt, 32, '\0'),
                    $text,
                    MCRYPT_MODE_ECB,
                    mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
                )
            )
        );
    }

    public function zeedhiDecrypt($text, $salt = '1460_loginManagement')
    {
        return trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                str_pad($salt, 32, '\0'),
                base64_decode($text),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
    }

    public static function getTokenLifeTimeParameter() {
        return InstanceManager::getInstance()->getParameter('tokenLifeTime');
    }

    public static function getDefaultEmailParameter() {
        return InstanceManager::getInstance()->getParameter('defaultEmail');
    }

    public static function getVariableMeasureEmailToSendParameter() {
        return InstanceManager::getInstance()->getParameter('variableMeasureEmailToSend');
    }

    public static function getOrgsAllMenusVisibleParameter() {
        return InstanceManager::getInstance()->getParameter('orgsAllMenusVisible');
    }

    public static function getTmpPathParameter() {
        return InstanceManager::getInstance()->getParameter('tmp_path');
    }

    public static function getSocketServerAddressParameter() {
        return InstanceManager::getInstance()->getParameter('socketServerAddress');
    }

    public static function getServerNameParameter() {
        return InstanceManager::getInstance()->getParameter('serverName');
    }

    public static function getMongoDate($date, $format = 'd/m/Y') {
        if(!($date instanceof \DateTime)) {
            $timezone = new \DateTimeZone('UTC');
            $date = \DateTime::createFromFormat($format, $date, $timezone);
        }
        return new \MongoDate($date->getTimestamp());
    }

    public static function isAssociativeArray($var) {
        return is_array($var) && array_diff_key($var, array_keys(array_keys($var)));
    }

    public static function getReversedDate($date, $inFormat = 'd/m/Y', $outFormat = 'Ymd') {
        $date = \DateTime::createFromFormat($inFormat, $date);
        return $date->format($outFormat);
    }

    public static function startsWith($haystack, $needle) {
        $length = strlen($needle);
        return (substr($haystack, 0, $length) === $needle);
    }

    public static function endsWith($haystack, $needle) {
        $length = strlen($needle);
        if ($length == 0) {
            return true;
        }
        return (substr($haystack, -$length) === $needle);
    }
} 